//  Data Structures with Java by John R. Hubbard
//  Copyright McGraw-Hill, 2001
//  Example 7.4 on page 127
//  An ArrayQueue class

import java.util.*;

public class ArrayQueue extends AbstractQueue
{ protected Object[] objects;
  protected int front=0;
  protected int back=0;
  protected int capacity=16;
  // INVARIANTS: objects[i] == null for 0 <= i < front;
  //             objects[i] != null for front <= i < back;
  //             objects[i] == null for back <= i < capacity;

  public ArrayQueue()
  { objects = new Object[capacity];
  }

  public ArrayQueue(int capacity)
  { this.capacity = capacity;
    objects = new Object[capacity];
  }

  public Object dequeue()
  { if (isEmpty()) throw new NoSuchElementException("Queue is empty");
    Object object = objects[front++];
    if (2*front>=capacity)  // shift left
    { for (int i=0; i<size(); i++)
        objects[i] = objects[i+front];
      back -= front;
      front = 0;
    }
    return object;
  }

  public Object enqueue(Object object)
  { if (back>=capacity)
    { Object[] temp = objects;
      capacity *= 2;  // double the capacity
      objects = new Object[capacity];
      for (int i=0; i<back-front; i++)
        objects[i] = temp[i+front];
      back -= front;
      front = 0;
    }
    objects[back++] = object;
    return object;
  }
  public Object getBack()
  { if (isEmpty()) throw new NoSuchElementException("Queue is empty");
    return objects[back-1];
  }

  public Object getFront()
  { if (isEmpty()) throw new NoSuchElementException("Queue is empty");
    return objects[front];
  }

  public Iterator iterator()
  { return new Iterator()  // anonymous inner class
    { private int cursor=front;
      public boolean hasNext()
      { return cursor<back;
      }
      public Object next()
      { if (cursor>=back) throw new NoSuchElementException();
        return objects[cursor++];
      }
      public void remove()
      { throw new UnsupportedOperationException();
      }
    };
  }

  public int size()
  { return back-front;
  }
}
