//  Data Structures with Java by John R. Hubbard
//  Copyright McGraw-Hill, 2001
//  Problem 7.9 on page 141

import schaums.dswj.Queue;

public class Pr0709
{ private static final int NUMBER_OF_SERVERS = 4;
  private static final double MEAN_INTERARRIVAL_TIME = 20.0;
  private static final int DURATION = 100;
  private static Server[] servers = new Server[NUMBER_OF_SERVERS];
  private static Queue clients = new ArrayQueue();
  private static Random random = new Random(MEAN_INTERARRIVAL_TIME);
  private static int totalTime, queueTime, totalNumberOfJobs,
                     totalServiceTime;
  private static int[] serviceTime = new int[NUMBER_OF_SERVERS];
  private static int[] numberOfJobs = new int[NUMBER_OF_SERVERS];

  public static void main(String[] args)
  { for (int i=0; i<NUMBER_OF_SERVERS; i++)
    { servers[i] = new Server();
      serviceTime[i] = numberOfJobs[i] = 0;
    }
    int timeOfNextArrival = random.intNextExponential();
    for (int t=0; t<DURATION; t++)
    { if (t == timeOfNextArrival)
      { clients.enqueue(new Client(t));
        timeOfNextArrival += random.intNextExponential();
      }
      for (int i=0; i<NUMBER_OF_SERVERS; i++)
      { Server server = servers[i];
        if (server.isFree())
        { if(!clients.isEmpty())
          { server.beginServing((Client)clients.dequeue(),t);
          }
        }
        else // server is serving a client
        { ++totalTime;  // counting time in system
          if (t == server.getTimeServiceEnds())
          { Client client = server.getClient();
            server.endServing(t);
            int waitTime = client.getWaitTime();
            serviceTime[i] = client.getServiceTime();
            ++numberOfJobs[i];  // count only those jobs that finish
          }
        }
      }
      totalTime += clients.size();
      queueTime += clients.size();
    }
    for (int i=0; i<NUMBER_OF_SERVERS; i++)
    { totalServiceTime += serviceTime[i];
      totalNumberOfJobs += numberOfJobs[i];
    }
    printStatistics();
  }

  private static void printStatistics()
  { System.out.println("Mean interarrival time: " + 
                        MEAN_INTERARRIVAL_TIME);
    System.out.println("Mean service time: "
      + Client.MEAN_JOB_SIZE);
    System.out.println("Number of servers: " + NUMBER_OF_SERVERS);
    System.out.println("Average number of jobs in the system: "
      + (float)totalTime/DURATION);
    System.out.println("Average number of jobs in the queue: "
      + (float)queueTime/DURATION);
    System.out.println("Average time a job spent in the system: "
      + (float)totalTime/totalNumberOfJobs);
    System.out.println("Average time a job spent in the queue: "
      + (float)queueTime/totalNumberOfJobs);
    System.out.println("Total time spent by all jobs in system: "
      + totalTime);
    System.out.println("Total time spent by all jobs in queue:  "
      + queueTime);
    System.out.println("Duration:                       " + DURATION);
    System.out.println("Total number of jobs completed: "
      + totalNumberOfJobs);
    System.out.println("Average service time for server:");
    for (int i=0; i<NUMBER_OF_SERVERS; i++)
      System.out.println("\t" + servers[i] + ": "
        + (float)serviceTime[i]/numberOfJobs[i]);
    System.out.println("Average service time among all jobs: "
      + (float)totalServiceTime/totalNumberOfJobs);
    System.out.println("Percent idle time for server:");
    for (int i=0; i<NUMBER_OF_SERVERS; i++)
      System.out.println("\t" + servers[i] + ": "
        + (float)(100.0*(DURATION - serviceTime[i]))/DURATION + "%");
  }
}

